/*
 * injection.c
 * Liberté
 *
 * Copyright (c) 2005 Landon J. Fuller <landonf@opendarwin.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holder nor the names of contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <mach/mach.h>

#include <sys/types.h>
#include <sys/uio.h>
#include <unistd.h>


#include "Shout.h"
#include "mach_inject.h"
#include "mach_override.h"


#include "mDNSHooks.h"
#include "libcHooks.h"


/*
 * mDNSResponder Overrides.
 * Keep in sync with mDNSHooks.h
 */

void INJECT_ENTRY(ptrdiff_t codeOffset, void *paramBlock, size_t paramSize) {
	mach_error_t err;
	const void *override;
	ShoutConfig *config;
	void (*func);
	
	config = (ShoutConfig *) paramBlock;
	if(paramSize != sizeof(ShoutConfig)) {
		/* Something is very wrong, try and alert the user and put ourselves to sleep ... */
		write(STDERR_FILENO, "paramSize != sizeof(ShoutConfig)\n", 33);
		thread_suspend(mach_thread_self());
	}
	
	/*
	 * Initialization
	 * XXX There's a race between the call to mach_override and the initialization.
	 * Should move the mach_override call to the initialization functions.
	 */
	
	/* Initialize mDNS Hooks */
	mDNSHooksInitialize(config);
	
	/* Initialize Standard C Library Hooks */
	libcHooksInitialize(config);
	
	/*
	 * Libc Hooks
	 */
	 
	/* setsockopt() */
	override = (const void *) &setsockoptOverride;
	override += codeOffset;
	err = mach_override(setsockoptSymbol, NULL, override, &func);
	setsockoptInitialize(func);
	
	/*
	 * mDNSHooks
	 */
	
	/* mDNS_Execute() */
	override = (const void *) &mDNS_ExecuteOverride;
	override += codeOffset;
	err = mach_override(mDNS_ExecuteSymbol, NULL, override, &func);
	mDNS_ExecuteInitialize(func);
	 
	/* mDNSAddrIsDNSMulticast() */
	override = (const void *) &mDNSAddrIsDNSMulticastOverride;
	override += codeOffset;
	err = mach_override(mDNSAddrIsDNSMulticastSymbol, NULL, override, &func);
	mDNSAddrIsDNSMulticastInitialize(func);
	
    /* mDNSPlatformSendUDP() */
	override = (const void *) &mDNSPlatformSendUDPOverride;
	override += codeOffset;
	err = mach_override(mDNSPlatformSendUDPSymbol, NULL, override, &func);
	mDNSPlatformSendUDPInitialize(func);
	
	thread_suspend(mach_thread_self());
}