/*
 * libcHooks.c
 * Liberté
 *
 * Copyright (c) 2005 Landon J. Fuller <landonf@opendarwin.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holder nor the names of contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <errno.h>

#include "libcHooks.h"

/*
 * Global Initialization
 */

static ShoutConfig *config;

void libcHooksInitialize(ShoutConfig *configPtr) {
	config = configPtr;
}

/*
 * setsockopt()
 * We hook setsockopt() to join mDNS sockets to our multicast group.
 * This is rather near the metal to be inserting ourselves, but there
 * are scant other locations in mDNS where we can do this.
 */

static setsockoptPtr setsockoptOrig;

void setsockoptInitialize(setsockoptPtr func) {
	setsockoptOrig = func;
}

int setsockoptOverride(int s, int level, int optname, const void *optval, socklen_t *optlen) {
	struct ip_mreq ourmreq;
	struct ip_mreq *mreq;
	socklen_t len;

	/* Dare we interfer? */
	if (level != IPPROTO_IP || optname != IP_ADD_MEMBERSHIP) {
		return(setsockoptOrig(s, level, optname, optval, optlen));
	}
	
	mreq = (struct ip_mreq *) optval;
	
	ourmreq.imr_interface.s_addr = mreq->imr_interface.s_addr;
	ourmreq.imr_multiaddr.s_addr = config->multicastGroup.v4.s_addr;
	len = sizeof(ourmreq);
	
	if (setsockoptOrig(s, IPPROTO_IP, IP_ADD_MEMBERSHIP, (const void *) &ourmreq, &len) != 0) {
		printf("Internal Liberte Error, %s, %d: setsockopt(%d, IPPROTO_IP, IP_ADD_MEMBERSHIP &ourmreq, &len): %s\n", __FUNCTION__, __LINE__, s,  strerror(errno));
	}

	return(setsockoptOrig(s, level, optname, optval, optlen));
}